@@ -1,18 +1,19 @@
 #! /usr/bin/env python
-# $Header: /cvsroot/pywebsvcs/zsi/ZSI/TC.py,v 1.27 2004/03/10 19:07:06 blunck2 Exp $
+# $Header: /cvsroot/pywebsvcs/zsi/ZSI/TC.py,v 1.38 2005/02/15 17:44:48 rsalz Exp $
 '''General typecodes.
 '''
 
 from ZSI import _copyright, _children, _child_elements, \
         _floattypes, _stringtypes, _seqtypes, _find_arraytype, _find_href, \
-        _find_encstyle, _textprotect, _textunprotect, \
+        _find_encstyle, _textprotect, \
         _find_xsi_attr, _find_type, _Node, \
         EvaluateException, _valid_encoding
+
+from ZSI.wstools.Namespaces import SCHEMA, SOAP
 try:
     from xml.dom.ext import Canonicalize
-    from xml.ns import SCHEMA, SOAP
 except:
-    from ZSI.compat import Canonicalize, SCHEMA, SOAP
+    from ZSI.compat import Canonicalize
 
 import re, types, time
 
@@ -25,6 +26,18 @@ _is_xsd_or_soap_ns = lambda ns: ns in [
                         SCHEMA.XSD3, SOAP.ENC, SCHEMA.XSD1, SCHEMA.XSD2, ]
 _find_nil = lambda E: _find_xsi_attr(E, "null") or _find_xsi_attr(E, "nil")
 
+
+def _get_object_id(pyobj):
+    x = id(pyobj)
+    # Python 2.3.x will generate a FutureWarning for negative IDs, so
+    # we use a different prefix character to ensure uniqueness, and
+    # call abs() to avoid the warning.
+    if x < 0:
+        return 'x%x' % abs(x)
+    else:
+        return 'o%x' % x
+
+
 class TypeCode:
     '''The parent class for all parseable SOAP types.
     Class data:
@@ -144,7 +157,9 @@ class TypeCode:
 
         # Parse the QNAME.
         list = type.split(':')
-        if len(list) != 2:
+        if len(list) == 1:
+            list = [None,list[0]]
+        elif len(list) != 2:
             raise EvaluateException('Malformed type attribute (not two colons)',
                     ps.Backtrace(elt))
         uri = ps.GetElementNSdict(elt).get(list[0])
@@ -266,16 +281,18 @@ class Any(TypeCode):
     def serialize(self, sw, pyobj, name=None, attrtext='', rpc=None, **kw):
         # What is attrtext for? It isn't used in the function.
         if hasattr(pyobj, 'typecode'):
+            if isinstance(pyobj.typecode, Any):
+                raise EvaluateException, 'Any can\'t serialize Any'
             pyobj.typecode.serialize(sw, pyobj, **kw)
             return
 
-        n = name or self.oname or rpc or 'E%x' % id(pyobj)
+        n = name or self.oname or rpc or 'E%s' % _get_object_id(pyobj)
         if self.ons:
             n = '%s:%s' % (self.ons, n)
         kw['name'] = n
         tc = type(pyobj)
         if tc == types.DictType or self.aslist:
-            if rpc != None:
+            if rpc is not None:
                 print >>sw, '<%s>' % rpc
             else:
                 if type(pyobj) != types.InstanceType:
@@ -288,7 +305,7 @@ class Any(TypeCode):
                 for key,val in pyobj.items():
                     Any(pname=key).serialize(sw, val)
 
-            if rpc != None:
+            if rpc is not None:
                 print >>sw, '</%s>' % rpc
             else:
                 if type(pyobj) != types.InstanceType:
@@ -318,7 +335,8 @@ class Any(TypeCode):
             serializer = Any.serialmap.get(tc)
             if not serializer and isinstance(pyobj, time.struct_time):
                 from ZSI.TCtimes import gDateTime
-                serializer = gDateTime();
+                serializer = gDateTime()
+
         if not serializer:
             # Last-chance; serialize instances as dictionary
             if type(pyobj) != types.InstanceType:
@@ -327,11 +345,15 @@ class Any(TypeCode):
             self.serialize(sw, pyobj.__dict__, **kw)
         else:
             # Try to make the element name self-describing
-            tag = getattr(serializer, 'tag', None)
-            if tag:
-                if tag.find(':') == -1: tag = 'SOAP-ENC:' + tag
-                kw['name'] = kw['oname'] = tag
-                kw['typed'] = 0
+            if not name and not self.oname:
+                tag = getattr(serializer, 'tag', None)
+                if tag:
+                    if tag.find(':') == -1: tag = 'SOAP-ENC:' + tag
+                    kw['name'] = kw['oname'] = tag
+                    kw['typed'] = 0
+            else:
+                # If element name is specified, must provide xsi:type
+                kw['typed'] = 1
             serializer.serialize(sw, pyobj, **kw)
 
 
@@ -375,7 +397,7 @@ class Void(TypeCode):
         return None
 
     def serialize(self, sw, pyobj, name=None, attrtext='', **kw):
-        n = name or self.oname or ('E%x' % id(pyobj))
+        n = name or self.oname or ('E%s' % _get_object_id(pyobj))
         print >>sw, '''<%s%s xsi:nil="1"/>''' % (n, attrtext)
 
 class String(TypeCode):
@@ -411,11 +433,10 @@ class String(TypeCode):
         if len(_children(elt)) == 0: return ''
         v = self.simple_value(elt, ps)
         if self.strip: v = v.strip()
-        if self.textprotect: v = _textunprotect(v)
         return v
 
     def serialize(self, sw, pyobj, name=None, attrtext='', orig=None, **kw):
-        objid = '%x' % id(pyobj)
+        objid = _get_object_id(pyobj)
         n = name or self.oname or ('E' + objid)
         if type(pyobj) in _seqtypes:
             print >>sw, '<%s%s href="%s"/>' % (n, attrtext, pyobj[0])
@@ -436,8 +457,17 @@ class String(TypeCode):
         else:
             idstr = ' id="%s"' % objid
         if self.textprotect: pyobj = _textprotect(pyobj)
+
+        # ignore the xmlns if it was explicitly stated
+        i = n.find('xmlns')
+        if i > 0:
+            ctag = '</%s>' % n[:i - 1]
+        else:
+            ctag = '</%s>' % n
+
         print >>sw, \
-            '<%s%s%s%s>%s</%s>' % (n, attrtext, idstr, tstr, pyobj, n)
+            '<%s%s%s%s>%s%s' % (n, attrtext, idstr, tstr, pyobj, ctag)
+
 
 class URI(String):
     '''A URI.
@@ -452,6 +482,17 @@ class URI(String):
     def serialize(self, sw, pyobj, name=None, attrtext='', **kw):
         String.serialize(self, sw, urlencode(pyobj), orig=pyobj, **kw)
 
+class QName(String):
+    '''A QName type
+    '''
+    parselist = [ (None,'QName') ]
+    tag = 'QName'
+
+class Token(String):
+    '''an xsd:token type
+    '''
+    parselist = [ (None, 'token') ]
+    tag = 'token'
 
 class Base64String(String):
     '''A Base64 encoded string.
@@ -588,7 +629,7 @@ class Integer(TypeCode):
         return v
 
     def serialize(self, sw, pyobj, name=None, attrtext='', **kw):
-        n = name or self.oname or ('E%x' % id(pyobj))
+        n = name or self.oname or ('E%s' % _get_object_id(pyobj))
         if kw.get('typed', self.typed):
             tstr = ' xsi:type="xsd:%s"' % (self.tag or 'integer')
         else:
@@ -718,7 +759,7 @@ class Decimal(TypeCode):
         return fp
 
     def serialize(self, sw, pyobj, name=None, attrtext='', **kw):
-        n = name or self.oname or ('E%x' % id(pyobj))
+        n = name or self.oname or ('E%s' % _get_object_id(pyobj))
         if kw.get('typed', self.typed):
             tstr = ' xsi:type="xsd:%s"' % (self.tag or 'decimal')
         else:
@@ -759,7 +800,7 @@ class Boolean(TypeCode):
         return 0
 
     def serialize(self, sw, pyobj, name=None, attrtext='', **kw):
-        n = name or self.oname or ('E%x' % id(pyobj))
+        n = name or self.oname or ('E%s' % _get_object_id(pyobj))
         if kw.get('typed', self.typed):
             tstr = ' xsi:type="xsd:boolean"'
         else:
@@ -817,7 +858,7 @@ class XML(TypeCode):
             Canonicalize(pyobj, sw, unsuppressedPrefixes=unsuppressedPrefixes,
                 comments=self.comments)
             return
-        objid = '%x' % id(pyobj)
+        objid = _get_object_id(pyobj)
         n = name or self.oname or ('E' + objid)
         if type(pyobj) in _stringtypes:
             print >>sw, '<%s%s href="%s"/>' % (n, attrtext, pyobj)
@@ -829,7 +870,7 @@ class XML(TypeCode):
 
     def cb(self, sw, pyobj, unsuppressedPrefixes=[]):
         if sw.Known(pyobj): return
-        objid = '%x' % id(pyobj)
+        objid = _get_object_id(pyobj)
         n = self.pname or ('E' + objid)
         print >>sw, '<%s SOAP-ENC:encodingStyle="" id="%s">' % (n, objid)
         Canonicalize(pyobj, sw, unsuppressedPrefixes=unsuppressedPrefixes,
