@@ -3,14 +3,14 @@
 # See LBNLCopyright for copyright notice!
 ###########################################################################
 
-# $Id: wsdlInterface.py,v 1.20 2004/02/06 04:04:40 tyger23 Exp $
+# $Id: wsdlInterface.py,v 1.36 2005/02/15 17:44:52 rsalz Exp $
 
-import re
+import re, sys
 import ZSI
 from ZSI.typeinterpreter import BaseTypeInterpreter
-from ZSI.wstools.Utility import Collection
+from ZSI.wstools.Utility import Collection, SplitQName
+from ZSI.wstools.Namespaces import SCHEMA
 import xml
-from xml.dom.ext import SplitQName
 
 ###########################################################################
 # "Virtual" superclasses for the wsdl adapters
@@ -332,8 +332,9 @@ class PartInterface:
     def __init__(self, part):
         self._part = part
 
-    def getName(self):
-        """return name of part, or None
+    def getName(self, mangle=True):
+        """returns mangled name for use w/python attributes (aname),
+        or the actual name for use in XML instances (pname,etc).
         """
         raise NotImplementedError, 'abstract method not implemented'
 
@@ -427,8 +428,9 @@ class SchemaTypeInterface:
     def __init__(self, type):
         self._type = type
 
-    def getName(self):
-        """get the type name
+    def getName(self, mangle=True):
+        """returns mangled name for use w/python attributes (aname),
+        or the actual name for use in XML instances (pname,etc).
         """
         raise NotImplementedError, 'abstract method not implemented'
 
@@ -502,8 +504,9 @@ class SchemaDefinitionInterface:
         """
         raise NotImplementedError, 'abstract method not implemented'
 
-    def getName(self):
-        """get name
+    def getName(self, mangle=True):
+        """returns mangled name for use w/python attributes (aname),
+        or the actual name for use in XML instances (pname,etc).
         """
         raise NotImplementedError, 'abstract method not implemented'
 
@@ -588,6 +591,11 @@ class SchemaDeclarationInterface:
         """
         raise NotImplementedError, 'abstract method not implemented'
 
+    def isXSD(self):
+        """boolean - checks if it's defined in the xsd ns
+        """
+        raise NotImplementedError, 'abstract method not implemented'
+
     def isElementReference(self):
         """boolean - an element ref?
         """
@@ -807,7 +815,7 @@ class ZSIPortAdapter(AdapterBase, PortInterface):
         self._ws = ws
         
     def getName(self):
-        return self._port.name
+        return self.mangle(self._port.name)
     
     def getExtensions(self):
         """returns extensions obj
@@ -1082,10 +1090,13 @@ class ZSIPartAdapter(AdapterBase, PartInterface):
         PartInterface.__init__(self, p)
         self._ws = ws
         
-    def getName(self):
-        """return name of part, or None
+    def getName(self, mangle=True):
+        """returns mangled name for use w/python attributes (aname),
+        or the actual name for use in XML instances (pname,etc).
         """
-        return (self._part.name or None)
+        if mangle is True:
+            return (self.mangle(self._part.name) or None)
+        return self._part.name or None
 
     def getElement(self):
         """return part's element or None
@@ -1113,7 +1124,7 @@ class ZSITypeAdapter(AdapterBase, TypeInterface):
     def getName(self):
         """return name of type, or None
         """
-        return self._type[1]
+        return self.mangle(self._type[1])
 
     def getTargetNamespace(self):
         """return targetnamespace of type
@@ -1156,11 +1167,11 @@ class ZSITypeAdapter(AdapterBase, TypeInterface):
         """
         
         pythontypes = {
-            'int'   : 'types.IntType',
-            'float' : 'types.FloatType',
-            'str'   : 'types.StringType',
-            'tuple' : 'types.TupleType',
-            'list'  : 'types.ListType',
+            'int'   : 'int',
+            'float' : 'float',
+            'str'   : 'basestring',
+            'tuple' : 'tuple',
+            'list'  : 'list',
             }
 
 
@@ -1359,9 +1370,19 @@ class ZSISchemaTypeAdapter(AdapterBase, SchemaTypeInterface):
                     ctype = ZSISchemaDefinitionAdapter(tp.content)
                     #ctype._def.attributes['xsd'] = tp.attributes['xsd']
                     return ctype
+                if isinstance( tp.content,
+                               ZSI.wstools.XMLSchema.AnonymousSimpleType):
+                    # XXX: fully dubious!
+                    # here i make an 'compress' an anonymous simple type
+                    # and the element declaration around it.
+                    stype = ZSISchemaDefinitionAdapter(tp.content)
+                    stype._def.attributes = tp.attributes
+                    return stype
+                
                 # otherwise...
                 if not tp.content:
                     return None
+
                 for c in tp.content:
                     if c.isDefinition():
                         return self.__adapterWrap(c)
@@ -1371,14 +1392,15 @@ class ZSISchemaTypeAdapter(AdapterBase, SchemaTypeInterface):
         else:
             raise TypeError, 'unknown adapter type: %s' % tp
 
-    def getName(self):
-        """get the type name
+    def getName(self, mangle=True):
+        """returns mangled name for use w/python attributes (aname),
+        or the actual name for use in XML instances (pname,etc).
         """
 
-        if self._type.attributes.has_key('name'):
+        name = self._type.attributes.get('name', None)
+        if name is not None and mangle is True:
             return self.mangle(self._type.attributes['name'])
-        else:
-            return None
+        return name
 
     def getTargetNamespace(self):
         """get the ns
@@ -1461,38 +1483,42 @@ class ZSISchemaDefinitionAdapter(AdapterBase, SchemaDefinitionInterface):
         if isinstance( tp, ZSI.wstools.XMLSchema.ComplexType ) or \
                isinstance( tp, ZSI.wstools.XMLSchema.SimpleType ):
             return ZSISchemaDefinitionAdapter(tp)
-        elif isinstance( tp, ZSI.wstools.XMLSchema.ElementDeclaration ) or \
+        if isinstance( tp, ZSI.wstools.XMLSchema.ElementDeclaration ) or \
              isinstance( tp, ZSI.wstools.XMLSchema.ElementWildCard ) or \
              isinstance( tp, ZSI.wstools.XMLSchema.\
                          LocalElementDeclaration )or \
              isinstance(tp, ZSI.wstools.XMLSchema.\
                         LocalAttributeDeclaration) or \
-             isinstance(tp, ZSI.wstools.XMLSchema.AttributeReference):
+             isinstance(tp, ZSI.wstools.XMLSchema.AttributeReference) or \
+             isinstance(tp, ZSI.wstools.XMLSchema.AttributeWildCard):
             return ZSISchemaDeclarationAdapter(tp)
         if isinstance( tp, ZSI.wstools.XMLSchema.ElementReference ):
             return ZSIElementReferenceAdapter(tp.getElementDeclaration('ref'),
                                               tp)
-        else:
-            raise TypeError, 'unknown adapter type: %s' % tp
+        if isinstance( tp, ZSI.wstools.XMLSchema.AttributeGroupReference ):
+            # we are currently not handling attributes...more soon...
+            return None
+        raise WsdlInterfaceLimitation, 'Limitation, can\'t handle XML Schema Item: %s' % tp.getItemTrace()
 
     def isDefinition(self):
         return True
 
     def isDeclaration(self):
         return False
-    
+
     def getTargetNamespace(self):
         """get NS
         """
         return self._def.getTargetNamespace()
 
-    def getName(self):
-        """get name
+    def getName(self, mangle=True):
+        """returns mangled name for use w/python attributes (aname),
+        or the actual name for use in XML instances (pname,etc).
         """
-        if self._def.attributes.has_key('name'):
+        name = self._def.attributes.get('name', None)
+        if name is not None and mangle is True:
             return self.mangle(self._def.attributes['name'])
-        else:
-            return None
+        return name
 
     def isComplexType(self):
         """boolean - complex type?
@@ -1539,9 +1565,17 @@ class ZSISchemaDefinitionAdapter(AdapterBase, SchemaDefinitionInterface):
                         # nested model groups , i am just 'flattening'
                         # the contents of the nested model group.
                         for x in c.content:
-                            group.append( self.__adapterWrap( x ) )
+                            if self.__adapterWrap( x ):
+                                t = self.__adapterWrap(x)
+                                if hasattr(t, '_ref'):
+                                    # it's an element ref then
+                                    t._ref.attributes['nillable'] = u"true"
+                                else:
+                                    t._dec.attributes['nillable'] = u"true"
+                                group.append( t )
                     else:
-                        group.append( self.__adapterWrap( c ) )
+                        if self.__adapterWrap( c ):
+                            group.append( self.__adapterWrap( c ) )
                 
             if isinstance( self._def.content,
                            ZSI.wstools.XMLSchema.All ):
@@ -1567,7 +1601,8 @@ class ZSISchemaDefinitionAdapter(AdapterBase, SchemaDefinitionInterface):
 
         if hasattr( self._def, 'attr_content' ) and self._def.attr_content:
             for a in self._def.attr_content:
-                attr.append( self.__adapterWrap(a) )
+                if self.__adapterWrap(a):
+                    attr.append( self.__adapterWrap(a) )
 
         return attr
 
@@ -1588,7 +1623,9 @@ class ZSISchemaDefinitionAdapter(AdapterBase, SchemaDefinitionInterface):
         """
         if isinstance(self._def, ZSI.wstools.XMLSchema.SimpleType) or \
                isinstance(self._def,
-                          ZSISchemaDeclarationAdapter.DefinitionContainer):
+                          ZSISchemaDeclarationAdapter.DefinitionContainer) or \
+               isinstance(self._def,
+                          ZSI.wstools.XMLSchema.AnonymousSimpleType):
             return True
         else:
             return False
@@ -1602,6 +1639,7 @@ class ZSISchemaDefinitionAdapter(AdapterBase, SchemaDefinitionInterface):
             if hasattr(self._def, 'content') and \
                    isinstance(self._def.content, ZSI.wstools.XMLSchema.\
                               SimpleType.Restriction):
+                
                 bti = self.__class__.bti
                 tpc = bti.get_typeclass( self._def.content.attributes['base'][1],
                                          self._def.content.attributes['base'][0])
@@ -1614,6 +1652,17 @@ class ZSISchemaDefinitionAdapter(AdapterBase, SchemaDefinitionInterface):
                     bti = self.__class__.bti
                     tpc = bti.get_typeclass( self._def.attributes['type'][1],
                                              self._def.attributes['type'][0] )
+                    
+            elif hasattr(self._def, 'content') and \
+                     isinstance(self._def.content,
+                                ZSI.wstools.XMLSchema.SimpleType.List):
+                # XXX: for xsd:list - nacient support
+                if self._def.content.attributes.has_key('itemType'):
+                    bti = self.__class__.bti
+                    tpc = bti.get_typeclass( self._def.content.\
+                                             attributes['itemType'][1],
+                                             self._def.content.\
+                                             attributes['itemType'][0] )
             elif self._def.content:
                 pass
 
@@ -1672,7 +1721,8 @@ class ZSISchemaDeclarationAdapter(AdapterBase, SchemaDeclarationInterface):
     def isWildCard(self):
         """boolean - wild card declaration?
         """
-        if isinstance(self._dec, ZSI.wstools.XMLSchema.ElementWildCard):
+        if isinstance(self._dec, ZSI.wstools.XMLSchema.ElementWildCard) or \
+               isinstance(self._dec, ZSI.wstools.XMLSchema.AttributeWildCard):
             return True
         else:
             return False
@@ -1691,11 +1741,18 @@ class ZSISchemaDeclarationAdapter(AdapterBase, SchemaDeclarationInterface):
         else:
             return False
 
+    def isXSD(self):
+        if self._dec.attributes.has_key('type'):
+            if self._dec.attributes['type'][0] in SCHEMA.XSD_LIST:
+                return True
+            
+        return False
+            
     def isElementReference(self):
         return False
 
     def containsLocalDefinition(self):
-        if self._dec.getTypeDefinition('type') == None:
+        if self._dec.getTypeDefinition('type') is None:
             if hasattr( self._dec, 'content' ):
                 if isinstance( self._dec.content,
                                ZSI.wstools.XMLSchema.LocalComplexType):
@@ -1703,6 +1760,12 @@ class ZSISchemaDeclarationAdapter(AdapterBase, SchemaDeclarationInterface):
 
         return False
 
+    def getLocalDefinition(self):
+        localDef = ZSISchemaDefinitionAdapter(self._dec.content)
+        # massage the contents of the object to match up with wsdl2python
+        localDef._def.attributes['name'] = self.getName()
+        return localDef
+
     def isLocalElement(self):
         """boolean - an element?
         """
@@ -1712,13 +1775,16 @@ class ZSISchemaDeclarationAdapter(AdapterBase, SchemaDeclarationInterface):
         else:
             return False
 
-    def getName(self):
-        """returns name
+    def getName(self, mangle=True):
+        """returns mangled name for use w/python attributes (aname),
+        or the actual name for use in XML instances (pname,etc).
         """
-        if self._dec.attributes.has_key('name'):
-            return self.mangle(self._dec.attributes['name'])
-        else:
-            return None
+        name = None
+        if hasattr(self._dec, 'attributes'):
+            name = self._dec.attributes.get('name', None)
+            if name and mangle is True:
+                return self.mangle(name)
+        return name
 
     def getTargetNamespace(self):
         """return namespace
@@ -1733,7 +1799,7 @@ class ZSISchemaDeclarationAdapter(AdapterBase, SchemaDeclarationInterface):
         
         typ = self._dec.getTypeDefinition('type')
 
-        if typ == None:
+        if typ is None:
             d = ZSISchemaDeclarationAdapter.DefinitionContainer()
             d.attributes = self._dec.attributes
             d.tns = self._dec.getTargetNamespace()
@@ -1774,6 +1840,14 @@ class ZSIElementReferenceAdapter(AdapterBase, ZSISchemaDeclarationAdapter):
     def isElementReference(self):
         return True
 
+    def getName(self):
+        if not ZSISchemaDeclarationAdapter.getName(self):
+            if hasattr( self._ref, 'attributes'):
+                if self._ref.attributes.has_key('ref'):
+                    return self._ref.attributes['ref'][1]
+        else:
+            return ZSISchemaDeclarationAdapter.getName(self)
+
     def getMinOccurs(self):
         """min occurs in local element dec
         """
@@ -1798,6 +1872,7 @@ class ZSIElementReferenceAdapter(AdapterBase, ZSISchemaDeclarationAdapter):
         else:
             return 0
 
+
 class ZSIModelGroupAdapter(AdapterBase, ModelGroupInterface):
     def isAll(self):
         """boolean - is complexType 'all'
@@ -1875,24 +1950,27 @@ class ZSIDerivedTypesAdapter(AdapterBase, DerivedTypesInterface):
             else:
                 raise WsdlInterfaceError, 'could not determine array type'
 
-            if t[0:4] == 'xsd:':
+            prefix, localName = SplitQName(t)
+            namespaceURI = self._content.getXMLNS(prefix)
+
+            if namespaceURI in SCHEMA.XSD_LIST:
                 isDefined = False
-                tns = 'http://www.w3.org/2001/XMLSchema'
                 bti = self.__class__.bti
-                atype = bti.get_typeclass( t[4:-2], tns )
+                atype = bti.get_typeclass( localName[:-2], namespaceURI )
             else:
                 isDefined = True
                 ns, atype = SplitQName(t)
                 atype = atype[:-2]
-            arrayinfo = ( t, atype, isDefined )
+                
+            arrayinfo = ( t, atype, isDefined, namespaceURI )
 
         else:
             raise WsdlInterfaceError, 'has no derivation base'
         
         if arrayinfo:
             return arrayinfo
-        else:
-            raise WsdlInterfaceError, 'could not determine array type'
+
+        raise WsdlInterfaceError, 'could not determine array type'
 
     def getDerivation(self):
 
